searchNodes=[{"doc":"ExState loads and persists workflow execution to a database through Ecto. The ExState.Execution is built through the subject&#39;s :workflow association. Setup defmodule ShipmentWorkflow do use ExState.Definition workflow &quot;shipment&quot; do subject :shipment, Shipment initial_state :preparing state :preparing do state :packing do on :packed, :sealing end state :sealing do on :unpack, :packing on :sealed, :sealed end state :sealed do final end on_final :shipping end state :shipping do on :shipped, :in_transit end state :in_transit do on :arrival, :arrived end state :arrived od on :accepted, :complete on :return, :returning end state :returning do on :arrival, :returned end state :returned do on :replace, :preparing end state :complete do final end end end defmodule Shipment do use Ecto.Schema use ExState.Ecto.Subject schema &quot;shipments&quot; do has_workflow ShipmentWorkflow end end Creating sale = %Sale{id: 1} execution = ExState.create(sale) #=&gt; %ExState.Execution{} Updating sale = %Sale{id: 1} {:ok, sale} = sale |&gt; ExState.load() |&gt; ExState.Execution.transition!(:packed) |&gt; ExState.Execution.transition!(:sealed) |&gt; ExState.persist() sale.workflow.state #=&gt; &quot;shipping&quot; {:error, reason} = ExState.transition(sale, :return) reason #=&gt; &quot;no transition from state shipping for event :return&quot;","ref":"ExState.html","title":"ExState","type":"module"},{"doc":"","ref":"ExState.html#complete/3","title":"ExState.complete/3","type":"function"},{"doc":"","ref":"ExState.html#create/1","title":"ExState.create/1","type":"function"},{"doc":"","ref":"ExState.html#create!/1","title":"ExState.create!/1","type":"function"},{"doc":"","ref":"ExState.html#create_multi/1","title":"ExState.create_multi/1","type":"function"},{"doc":"","ref":"ExState.html#decision/4","title":"ExState.decision/4","type":"function"},{"doc":"","ref":"ExState.html#load/1","title":"ExState.load/1","type":"function"},{"doc":"","ref":"ExState.html#persist/1","title":"ExState.persist/1","type":"function"},{"doc":"","ref":"ExState.html#transition/3","title":"ExState.transition/3","type":"function"},{"doc":"ExState.Definition provides macros to define a workflow state chart. A workflow is defined with a name: workflow &quot;make_deal&quot; do #... end Subject The subject of the workflow is used to associate the workflow for lookup in the database. The subject is added to the context under the defined key and can be used in callbacks use_step?/2, and guard_transition/3. Subject names and types are defined using the subject keyword: subject :deal, Deal Initial State A workflow must have an initial state: initial_state :pending This state must be defined using a seperate state definition. States States have a name, and optional sub-states, steps, and transitions: state :pending do initial_state :preparing state :preparing do on :review, :reviewing end state :reviewing do on :cancel, :cancelled end end state :cancelled Transitions may be a list of targets, in which case the first target state which is allowed by guard_transition/3 will be used. state :pending do initial_state :preparing state :preparing do on :prepared, [:reviewing, :sending] end state :reviewing do on :cancel, :cancelled end state :sending do on :send, :sent end end def guard_transition(shipment, :preparing, :reviewing) do if shipment.requires_review? do :ok else {:error, &quot;no review required&quot;} end end def guard_transition(shipment, :preparing, :sending) do if shipment.requires_review? do {:error, &quot;review required&quot;} else :ok end end def guard_transition(_, _, ), do: :ok Transitions may also use the null event, which occurs immediately on entering a state. This is useful determining the initial state dynamically. state :unknown do on :_, [:a, :b] end state :a state :b def guard_transition(order, :unknown, :a), do if order.use_a?, do: :ok, else: {:error, :use_b} end Steps Steps must be completed in order of definition: state :preparing do step :read step :sign step :confirm end Steps can be defined in parallel, meaning any step from the block can be completed independent of order: state :preparing do parallel do step :read step :sign step :confirm end end Step completed events can be handled to transition to new states: state :preparing do step :read step :sign step :confirm on_completed :confirm, :done end state :done States can be ignored on a subject basis through use_step/2: def use_step(:sign, %{deal: deal}) do deal.requires_signature? end def use_step(_, _), do: true Virtual States States definitions can be reused through virtual states: virtual :completion_states do state :working do step :read step :sign step :confirm end end state :completing_a do using :completion_states on_completed :confirm, :completing_b end state :completing_b do using :completion_states on_completed :confirm, :done end state :done Decisions Decisions are steps that have defined options. The selection of an option can be used to determine state transitions: state :preparing do step :read step :review_terms on_decision :review_terms, :accept, :signing on_decision :review_terms, :reject, :rejected end state :signing do step :sign on_completed :sign, :done end state :rejected state :done Transitions By default, transitions reference sibling states: state :one do on :done, :two end state :two Transitions can reference states one level up the heirarchy (a sibling of the parent state) by using {:&lt;, :state}, in the following form: state :one do state :a do on :done, {:&lt;, :two} end end state :two Transitions can also explicitly denote legal events in the current state using :_. The following adds a transition to the current state: state :one do on :done, :two end state :two do on :done, :_ end Transitions to the current state will reset completed steps in the current state by default. Step state can be preserved by using the reset: false option. state :one do step :a on :done, :two on :retry, :_, reset: true end state :two do step :b on :done, :_, reset: false end Guards Guards validate that certain dynamic conditions are met in order to allow state transitions: def guard_transition(:one, :two, %{note: note}) do if length(note.text) &gt; 5 do :ok else {:error, &quot;Text must be greater than 5 characters long&quot;} end end def guard_transition(_, _, _), do: :ok Execution will stop the state transition if {:error, reason} is returned from the guard, and will allow the transition if :ok is returned. Actions Actions are side effects that happen on events. Events can be transitions, entering a state, or exiting a state. state :one do on_entry :send_notification on_entry :log_activity on :done, :two, action: [:update_done_at] end state :two do step :send_something end def update_done_at(%{note: note} = context) do {:updated, Map.put(context, :note, %{note | done_at: now()})} end Actions can return a {:updated, context} tuple to add the updated context to the execution state. A default Execution.execute_actions/1 function is provided which executes triggered actions in a fire-and-forget fashion. See ExState.persist/1 for an example of transactionally executing actions. Actions should also not explicity guard state transitions. Guards should use guard_transition/3.","ref":"ExState.Definition.html","title":"ExState.Definition","type":"behaviour"},{"doc":"","ref":"ExState.Definition.html#c:guard_transition/3","title":"ExState.Definition.guard_transition/3","type":"callback"},{"doc":"","ref":"ExState.Definition.html#updated/1","title":"ExState.Definition.updated/1","type":"function"},{"doc":"","ref":"ExState.Definition.html#updated/2","title":"ExState.Definition.updated/2","type":"function"},{"doc":"","ref":"ExState.Definition.html#c:use_step?/2","title":"ExState.Definition.use_step?/2","type":"callback"},{"doc":"","ref":"ExState.Definition.html#workflow/2","title":"ExState.Definition.workflow/2","type":"macro"},{"doc":"","ref":"ExState.Definition.html#t:context/0","title":"ExState.Definition.context/0","type":"type"},{"doc":"","ref":"ExState.Definition.html#t:state/0","title":"ExState.Definition.state/0","type":"type"},{"doc":"","ref":"ExState.Definition.html#t:step/0","title":"ExState.Definition.step/0","type":"type"},{"doc":"","ref":"ExState.Definition.Chart.html","title":"ExState.Definition.Chart","type":"module"},{"doc":"","ref":"ExState.Definition.Chart.html#describe/1","title":"ExState.Definition.Chart.describe/1","type":"function"},{"doc":"","ref":"ExState.Definition.Chart.html#events/1","title":"ExState.Definition.Chart.events/1","type":"function"},{"doc":"","ref":"ExState.Definition.Chart.html#parent/2","title":"ExState.Definition.Chart.parent/2","type":"function"},{"doc":"","ref":"ExState.Definition.Chart.html#participant_names/1","title":"ExState.Definition.Chart.participant_names/1","type":"function"},{"doc":"","ref":"ExState.Definition.Chart.html#put_participant/2","title":"ExState.Definition.Chart.put_participant/2","type":"function"},{"doc":"","ref":"ExState.Definition.Chart.html#put_states/2","title":"ExState.Definition.Chart.put_states/2","type":"function"},{"doc":"","ref":"ExState.Definition.Chart.html#state/2","title":"ExState.Definition.Chart.state/2","type":"function"},{"doc":"","ref":"ExState.Definition.Chart.html#state/3","title":"ExState.Definition.Chart.state/3","type":"function"},{"doc":"","ref":"ExState.Definition.Chart.html#state_names/1","title":"ExState.Definition.Chart.state_names/1","type":"function"},{"doc":"","ref":"ExState.Definition.Chart.html#states/1","title":"ExState.Definition.Chart.states/1","type":"function"},{"doc":"","ref":"ExState.Definition.Chart.html#step_names/1","title":"ExState.Definition.Chart.step_names/1","type":"function"},{"doc":"","ref":"ExState.Definition.Chart.html#steps/1","title":"ExState.Definition.Chart.steps/1","type":"function"},{"doc":"","ref":"ExState.Definition.Chart.html#transitions/1","title":"ExState.Definition.Chart.transitions/1","type":"function"},{"doc":"","ref":"ExState.Definition.Chart.html#t:subject/0","title":"ExState.Definition.Chart.subject/0","type":"type"},{"doc":"","ref":"ExState.Definition.Chart.html#t:t/0","title":"ExState.Definition.Chart.t/0","type":"type"},{"doc":"","ref":"ExState.Definition.Compiler.html","title":"ExState.Definition.Compiler","type":"module"},{"doc":"","ref":"ExState.Definition.Compiler.html#compile/3","title":"ExState.Definition.Compiler.compile/3","type":"function"},{"doc":"","ref":"ExState.Definition.Compiler.Env.html","title":"ExState.Definition.Compiler.Env","type":"module"},{"doc":"","ref":"ExState.Definition.State.html","title":"ExState.Definition.State","type":"module"},{"doc":"","ref":"ExState.Definition.State.html#actions/2","title":"ExState.Definition.State.actions/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#add_action/3","title":"ExState.Definition.State.add_action/3","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#add_parallel_steps/2","title":"ExState.Definition.State.add_parallel_steps/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#add_repeatable_step/2","title":"ExState.Definition.State.add_repeatable_step/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#add_step/2","title":"ExState.Definition.State.add_step/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#add_step/3","title":"ExState.Definition.State.add_step/3","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#add_transition/2","title":"ExState.Definition.State.add_transition/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#append/2","title":"ExState.Definition.State.append/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#child/2","title":"ExState.Definition.State.child/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#child?/2","title":"ExState.Definition.State.child?/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#combine/1","title":"ExState.Definition.State.combine/1","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#complete_step/3","title":"ExState.Definition.State.complete_step/3","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#drop_last/1","title":"ExState.Definition.State.drop_last/1","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#filter_steps/2","title":"ExState.Definition.State.filter_steps/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#final?/1","title":"ExState.Definition.State.final?/1","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#id/1","title":"ExState.Definition.State.id/1","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#last/1","title":"ExState.Definition.State.last/1","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#name/1","title":"ExState.Definition.State.name/1","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#next_steps/1","title":"ExState.Definition.State.next_steps/1","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#parent/1","title":"ExState.Definition.State.parent/1","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#put_completed_step/3","title":"ExState.Definition.State.put_completed_step/3","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#repeatable?/2","title":"ExState.Definition.State.repeatable?/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#resolve/2","title":"ExState.Definition.State.resolve/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#sibling/2","title":"ExState.Definition.State.sibling/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#sibling?/2","title":"ExState.Definition.State.sibling?/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#split/1","title":"ExState.Definition.State.split/1","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#transition/2","title":"ExState.Definition.State.transition/2","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#transitions/1","title":"ExState.Definition.State.transitions/1","type":"function"},{"doc":"","ref":"ExState.Definition.State.html#t:state_type/0","title":"ExState.Definition.State.state_type/0","type":"type"},{"doc":"","ref":"ExState.Definition.State.html#t:t/0","title":"ExState.Definition.State.t/0","type":"type"},{"doc":"","ref":"ExState.Definition.Step.html","title":"ExState.Definition.Step","type":"module"},{"doc":"","ref":"ExState.Definition.Step.html#complete/2","title":"ExState.Definition.Step.complete/2","type":"function"},{"doc":"","ref":"ExState.Definition.Step.html#id/1","title":"ExState.Definition.Step.id/1","type":"function"},{"doc":"","ref":"ExState.Definition.Step.html#name/1","title":"ExState.Definition.Step.name/1","type":"function"},{"doc":"","ref":"ExState.Definition.Step.html#new/2","title":"ExState.Definition.Step.new/2","type":"function"},{"doc":"","ref":"ExState.Definition.Step.html#order/2","title":"ExState.Definition.Step.order/2","type":"function"},{"doc":"","ref":"ExState.Definition.Step.html#t:t/0","title":"ExState.Definition.Step.t/0","type":"type"},{"doc":"","ref":"ExState.Definition.Transition.html","title":"ExState.Definition.Transition","type":"module"},{"doc":"","ref":"ExState.Definition.Transition.html#new/3","title":"ExState.Definition.Transition.new/3","type":"function"},{"doc":"","ref":"ExState.Definition.Transition.html#t:event/0","title":"ExState.Definition.Transition.event/0","type":"type"},{"doc":"","ref":"ExState.Definition.Transition.html#t:t/0","title":"ExState.Definition.Transition.t/0","type":"type"},{"doc":"","ref":"ExState.Ecto.Migration.html","title":"ExState.Ecto.Migration","type":"module"},{"doc":"","ref":"ExState.Ecto.Migration.html#up/1","title":"ExState.Ecto.Migration.up/1","type":"function"},{"doc":"","ref":"ExState.Ecto.Model.html","title":"ExState.Ecto.Model","type":"module"},{"doc":"","ref":"ExState.Ecto.Model.html#new/2","title":"ExState.Ecto.Model.new/2","type":"function"},{"doc":"","ref":"ExState.Ecto.Model.html#put_assoc_maybe/4","title":"ExState.Ecto.Model.put_assoc_maybe/4","type":"function"},{"doc":"ExState.Ecto.Multi provides convencience functions for creating and transitioning workflows in Ecto.Multi operations. Example def create_sale() do Multi.new() |&gt; Multi.create(:sale, Sale.new()) |&gt; ExState.Ecto.Multi.create(:sale) |&gt; ExState.Ecto.Multi.transition(:sale, :packed) |&gt; Repo.transaction() end","ref":"ExState.Ecto.Multi.html","title":"ExState.Ecto.Multi","type":"module"},{"doc":"","ref":"ExState.Ecto.Multi.html#complete/4","title":"ExState.Ecto.Multi.complete/4","type":"function"},{"doc":"","ref":"ExState.Ecto.Multi.html#create/2","title":"ExState.Ecto.Multi.create/2","type":"function"},{"doc":"","ref":"ExState.Ecto.Multi.html#decision/5","title":"ExState.Ecto.Multi.decision/5","type":"function"},{"doc":"","ref":"ExState.Ecto.Multi.html#transition/4","title":"ExState.Ecto.Multi.transition/4","type":"function"},{"doc":"ExState.Ecto.Query provides functions for querying workflow state in the database through Ecto.","ref":"ExState.Ecto.Query.html","title":"ExState.Ecto.Query","type":"module"},{"doc":"","ref":"ExState.Ecto.Query.html#to_state_id/1","title":"ExState.Ecto.Query.to_state_id/1","type":"function"},{"doc":"","ref":"ExState.Ecto.Query.html#to_state_list/1","title":"ExState.Ecto.Query.to_state_list/1","type":"function"},{"doc":"","ref":"ExState.Ecto.Query.html#to_step_name/1","title":"ExState.Ecto.Query.to_step_name/1","type":"function"},{"doc":"where_any_state/2 takes a subject query and a state and filters based on workflows that are equal to or in a child state of the given state. Nested states can be passed as a list of states and will be converted to a valid state identifier in the query. Examples: investment #=&gt; %Investment{workflow: %Workflow{state: &quot;subscribing.confirming_options&quot;}} Investment |&gt; where_any_state(:subscribing) |&gt; Repo.all() #=&gt; [investment] Investment |&gt; where_any_state(&quot;subscribing&quot;) |&gt; Repo.all() #=&gt; [investment] Investment |&gt; where_any_state([:subscribing, :confirming_options]) |&gt; Repo.all() #=&gt; [investment] Investment |&gt; where_any_state(:resubmitting) |&gt; Repo.all() #=&gt; []","ref":"ExState.Ecto.Query.html#where_any_state/2","title":"ExState.Ecto.Query.where_any_state/2","type":"function"},{"doc":"where_state/2 takes a subject query and state and filters based on workflows that are in the exact state that is passed. Nested states can be passed as a list of states and will be converted to a valid state identifier in the query. Pass the state as the keyword list not: state in order to query the inverse. Examples: investment #=&gt; %Investment{workflow: %Workflow{state: &quot;subscribing.confirming_options&quot;}} Investment |&gt; where_state(&quot;subscribing.confirming_options&quot;) |&gt; Repo.all() #=&gt; [investment] Investment |&gt; where_state([:subscribing, :confirming_options]) |&gt; Repo.all() #=&gt; [investment] Investment |&gt; where_state([&quot;subscribing&quot;, &quot;confirming_options&quot;]) |&gt; Repo.all() #=&gt; [investment] Investment |&gt; where_state(:subscribing) |&gt; Repo.all() #=&gt; [] Investment |&gt; where_state(not: [:subscribing, :confirming_suitability]) |&gt; Repo.all() #=&gt; [investment] Investment |&gt; where_state(not: [:subscribing, :confirming_options]) |&gt; Repo.all() #=&gt; []","ref":"ExState.Ecto.Query.html#where_state/2","title":"ExState.Ecto.Query.where_state/2","type":"function"},{"doc":"where_state_in/2 takes a subject query and a list of states and filters based on workflows that are in one of the exact states that are passed. Nested states can be passed as a list of states and will be converted to a valid state identifier in the query. Pass the state as the keyword list not: state in order to query the inverse. Examples: investment1 #=&gt; %Investment{workflow: %Workflow{state: &quot;subscribing.confirming_options&quot;}} investment2 #=&gt; %Investment{workflow: %Workflow{state: &quot;executed&quot;}} Investment |&gt; where_state_in([ [:subscribing, :confirming_options], :executed ]) |&gt; Repo.all() #=&gt; [investment1, investment2] Investment |&gt; where_state_in([&quot;subscribing.confirming_options&quot;]) |&gt; Repo.all() #=&gt; [investment1] Investment |&gt; where_state_in([:subscribing]) |&gt; Repo.all() #=&gt; [] Investment |&gt; where_state_in(not: [[:subscribing, :confirming_options]]) |&gt; Repo.all() #=&gt; [investment2] Investment |&gt; where_state_in(not: [:subscribing]) |&gt; Repo.all() #=&gt; [investment1, investment2]","ref":"ExState.Ecto.Query.html#where_state_in/2","title":"ExState.Ecto.Query.where_state_in/2","type":"function"},{"doc":"","ref":"ExState.Ecto.Query.html#where_step_complete/2","title":"ExState.Ecto.Query.where_step_complete/2","type":"function"},{"doc":"","ref":"ExState.Ecto.Subject.html","title":"ExState.Ecto.Subject","type":"module"},{"doc":"","ref":"ExState.Ecto.Subject.html#has_workflow/1","title":"ExState.Ecto.Subject.has_workflow/1","type":"macro"},{"doc":"","ref":"ExState.Ecto.Subject.html#has_workflow/3","title":"ExState.Ecto.Subject.has_workflow/3","type":"macro"},{"doc":"","ref":"ExState.Ecto.Subject.html#put_workflow/2","title":"ExState.Ecto.Subject.put_workflow/2","type":"function"},{"doc":"","ref":"ExState.Ecto.Subject.html#workflow_assoc_name/1","title":"ExState.Ecto.Subject.workflow_assoc_name/1","type":"function"},{"doc":"","ref":"ExState.Ecto.Subject.html#workflow_definition/1","title":"ExState.Ecto.Subject.workflow_definition/1","type":"function"},{"doc":"","ref":"ExState.Ecto.Workflow.html","title":"ExState.Ecto.Workflow","type":"module"},{"doc":"","ref":"ExState.Ecto.Workflow.html#changeset/2","title":"ExState.Ecto.Workflow.changeset/2","type":"function"},{"doc":"","ref":"ExState.Ecto.Workflow.html#completed_steps/1","title":"ExState.Ecto.Workflow.completed_steps/1","type":"function"},{"doc":"","ref":"ExState.Ecto.Workflow.html#new/1","title":"ExState.Ecto.Workflow.new/1","type":"function"},{"doc":"","ref":"ExState.Ecto.Workflow.html#t:t/0","title":"ExState.Ecto.Workflow.t/0","type":"type"},{"doc":"","ref":"ExState.Ecto.WorkflowStep.html","title":"ExState.Ecto.WorkflowStep","type":"module"},{"doc":"","ref":"ExState.Ecto.WorkflowStep.html#changeset/2","title":"ExState.Ecto.WorkflowStep.changeset/2","type":"function"},{"doc":"","ref":"ExState.Ecto.WorkflowStep.html#new/1","title":"ExState.Ecto.WorkflowStep.new/1","type":"function"},{"doc":"","ref":"ExState.Ecto.WorkflowStep.html#put_completion/2","title":"ExState.Ecto.WorkflowStep.put_completion/2","type":"function"},{"doc":"","ref":"ExState.Ecto.WorkflowStep.html#t:t/0","title":"ExState.Ecto.WorkflowStep.t/0","type":"type"},{"doc":"ExState.Execution executes state transitions with a state chart.","ref":"ExState.Execution.html","title":"ExState.Execution","type":"module"},{"doc":"Completes a step and transitions the execution with {:completed, step_id} event.","ref":"ExState.Execution.html#complete/2","title":"ExState.Execution.complete/2","type":"function"},{"doc":"","ref":"ExState.Execution.html#complete!/2","title":"ExState.Execution.complete!/2","type":"function"},{"doc":"","ref":"ExState.Execution.html#complete?/1","title":"ExState.Execution.complete?/1","type":"function"},{"doc":"Continues a workflow execution from the specified state.","ref":"ExState.Execution.html#continue/2","title":"ExState.Execution.continue/2","type":"function"},{"doc":"","ref":"ExState.Execution.html#continue/3","title":"ExState.Execution.continue/3","type":"function"},{"doc":"","ref":"ExState.Execution.html#continue/4","title":"ExState.Execution.continue/4","type":"function"},{"doc":"Completes a decision and transitions the execution with {:decision, step_id, decision} event.","ref":"ExState.Execution.html#decision/3","title":"ExState.Execution.decision/3","type":"function"},{"doc":"","ref":"ExState.Execution.html#decision!/3","title":"ExState.Execution.decision!/3","type":"function"},{"doc":"Returns serializable data representing the execution.","ref":"ExState.Execution.html#dump/1","title":"ExState.Execution.dump/1","type":"function"},{"doc":"Executes the provided action name through the callback module.","ref":"ExState.Execution.html#execute_action/2","title":"ExState.Execution.execute_action/2","type":"function"},{"doc":"Executes any queued actions on the execution.","ref":"ExState.Execution.html#execute_actions/1","title":"ExState.Execution.execute_actions/1","type":"function"},{"doc":"","ref":"ExState.Execution.html#execute_actions!/1","title":"ExState.Execution.execute_actions!/1","type":"function"},{"doc":"","ref":"ExState.Execution.html#get_state/2","title":"ExState.Execution.get_state/2","type":"function"},{"doc":"","ref":"ExState.Execution.html#get_subject/1","title":"ExState.Execution.get_subject/1","type":"function"},{"doc":"Creates a new workflow execution from the initial state.","ref":"ExState.Execution.html#new/1","title":"ExState.Execution.new/1","type":"function"},{"doc":"","ref":"ExState.Execution.html#new/2","title":"ExState.Execution.new/2","type":"function"},{"doc":"","ref":"ExState.Execution.html#new/3","title":"ExState.Execution.new/3","type":"function"},{"doc":"","ref":"ExState.Execution.html#put_context/2","title":"ExState.Execution.put_context/2","type":"function"},{"doc":"","ref":"ExState.Execution.html#put_context/3","title":"ExState.Execution.put_context/3","type":"function"},{"doc":"","ref":"ExState.Execution.html#put_state/2","title":"ExState.Execution.put_state/2","type":"function"},{"doc":"","ref":"ExState.Execution.html#put_subject/2","title":"ExState.Execution.put_subject/2","type":"function"},{"doc":"","ref":"ExState.Execution.html#put_transition/2","title":"ExState.Execution.put_transition/2","type":"function"},{"doc":"Transitions execution with the event and returns a result tuple.","ref":"ExState.Execution.html#transition/2","title":"ExState.Execution.transition/2","type":"function"},{"doc":"","ref":"ExState.Execution.html#transition!/2","title":"ExState.Execution.transition!/2","type":"function"},{"doc":"Transitions execution with the event and returns updated or unchanged execution.","ref":"ExState.Execution.html#transition_maybe/2","title":"ExState.Execution.transition_maybe/2","type":"function"},{"doc":"","ref":"ExState.Execution.html#will_transition?/2","title":"ExState.Execution.will_transition?/2","type":"function"},{"doc":"Continues a workflow execution with the completed steps. Use in conjunction with continue to resume execution.","ref":"ExState.Execution.html#with_completed/4","title":"ExState.Execution.with_completed/4","type":"function"},{"doc":"","ref":"ExState.Execution.html#with_meta/3","title":"ExState.Execution.with_meta/3","type":"function"},{"doc":"","ref":"ExState.Execution.html#t:t/0","title":"ExState.Execution.t/0","type":"type"},{"doc":"","ref":"ExState.Result.html","title":"ExState.Result","type":"module"},{"doc":"","ref":"ExState.Result.html#flat_map/2","title":"ExState.Result.flat_map/2","type":"function"},{"doc":"","ref":"ExState.Result.html#get/1","title":"ExState.Result.get/1","type":"function"},{"doc":"","ref":"ExState.Result.html#map/2","title":"ExState.Result.map/2","type":"function"},{"doc":"","ref":"ExState.Result.Multi.html","title":"ExState.Result.Multi","type":"module"},{"doc":"","ref":"ExState.Result.Multi.html#extract/2","title":"ExState.Result.Multi.extract/2","type":"function"},{"doc":"ExState Elixir state machines, statecharts, and workflows for Ecto models.","ref":"readme.html","title":"ExState","type":"extras"},{"doc":"If available in Hex, the package can be installed by adding ex_state to your list of dependencies in mix.exs: def deps do [ {:ex_state_ecto, &quot;~&gt; 0.2.0&quot;} ] end Documentation can be generated with ExDoc and published on HexDocs. Once published, the docs can be found at https://hexdocs.pm/ex_state.","ref":"readme.html#installation","title":"ExState - Installation","type":"extras"},{"doc":"Without Ecto Example Ecto Setup defmodule MyApp.Repo.Migrations.AddWorkflows do def up do # Ensure Ecto.UUID support is enabled: execute(&quot;CREATE EXTENSION IF NOT EXISTS pgcrypto&quot;) ExState.Ecto.Migration.up() end def down do end end config :ex_state, ExState, repo: MyApp.Repo Defining States defmodule SaleWorkflow do use ExState.Definition alias MyApp.Repo workflow &quot;sale&quot; do subject :sale, Sale participant :seller participant :buyer initial_state :pending state :pending do on :send, :sent on :cancel, :cancelled end state :sent do parallel do step :acknowledge_receipt, participant: :buyer step :close, participant: :seller end on :cancelled, :cancelled on_completed :acknowledge_receipt, :receipt_acknowledged on_completed :close, :closed end state :receipt_acknowledged do step :close, participant: :seller on_completed :close, :closed end state :closed state :cancelled do on_entry :update_cancelled_at end end def guard_transition(:pending, :sent, %{sale: %{ready: false}}) do {:error, &quot;not ready&quot;} end def guard_transition(_from, _to, _context), do: :ok def update_cancelled_at(%{sale: sale}) do sale |&gt; Sale.changeset(%{cancelled_at: DateTime.utc_now()}) |&gt; Repo.update() end end defmodule Sale do use Ecto.Schema use ExState.Ecto.Subject import Ecto.Changeset schema &quot;sales&quot; do has_workflow SaleWorkflow field :product_id, :string field :cancelled_at, :utc_datetime end end Changing States sale |&gt; ExState.create() |&gt; ExState.Execution.transition_maybe(:send) |&gt; ExState.persist() sale |&gt; ExState.load() |&gt; ExState.Execution.transition_maybe(:cancelled) |&gt; ExState.persist() def create_sale(params) do Multi.new() |&gt; Multi.insert(:sale, Sale.new(params)) |&gt; ExState.Ecto.Multi.create(:sale) |&gt; Repo.transaction() end def cancel_sale(id, user_id: user_id) do sale = Repo.get(Sale, id) ExState.transition(sale, :cancel, user_id: user_id) end Completing Steps def acknowledge_receipt(id, user_id: user_id) do sale = Repo.get(Sale, id) ExState.complete(sale, :acknowledge_receipt, user_id: user_id) end Running Tests Setup test database MIX_ENV=test mix ecto.create mix test","ref":"readme.html#usage","title":"ExState - Usage","type":"extras"},{"doc":"Extract ex_state_core, and other backend / db packages. Multiple workflows per subject. Allow configurable primary key / UUID type for usage across different databases. Tracking event history with metadata. Add SCXML support Define schema for serialization / json API usage / client consumption. Parallel states History states","ref":"readme.html#todo","title":"ExState - TODO","type":"extras"}]